#contrastImage.py

#import dependencies
import numpy as np
import cv2
from . import histogram

__all__ = ['upcontrastImage']

"""
upcontrastImage - function to increase contrast of an image
Parameters:
	image		I/P	image need increasing contrast
	enhanced_image	O/P	image after being increased contrast
"""
def upcontrastImage(image):
	"""
	Perform Histogram equalization to strech images in order to increase contrast
	"""

	#compute histogram
	#hist, bins  = np.histogram(image.flatten(), bins = 256, range = (0, 255))
	hist = histogram.histogram(image)

	#equalize histogram
	colors = 256

	stretched_hist = histogram.eq_hist(hist, colors, image.size)

	#enhance image contrast
	enhanced_image = np.zeros(shape = image.shape)

	height = len(image)
	width = len(image[0])	
	for row in range(height):
		for col in range(width):
			enhanced_image[row, col] = stretched_hist[image[row, col]]

	return enhanced_image



#linear_scale_image.py

#import dependencies
import numpy as np
import math
from . import histogram

__all__ = ['linear_scale']

"""
linear_scale - function to scale images linearly
"""
def linear_scale(img, A = 0, B = 255):
	#by default, the A and B are 0 and 255 respectively
	#get a nd b (smallest and largest color value) of the image that b > a

	#flatten img
	old_range = np.unique(img)
	a = np.min(old_range)
	b = np.max(old_range)
	ba = b - a
	BA = B - A

	scaled_img = img.copy()
	coef = BA / ba

	for row in range(len(scaled_img)):
		for col in range(len(scaled_img[0])):
			scaled_img[row, col] = np.floor((img[row, col] - a)* coef) + A

	return scaled_img
	




#image_function.py
"""
Module: image_function - module to implement resize (zoom in /out), rotate, and flip images
Last udpated: Feb. 2, 2020
"""

#import dependencies
from . import geom_transform as gtf

"""
shift - function to shift images
Parameters:
	image	I/P	image input
	alpha	I/P	the num of spaces shift horizontally
	beta 	I/P	the num of spaces shift vertically
		O/P	shifted image
"""
def shift(image, alpha, beta):
	#call Translation algorithm from Geometric Transform to shfit images by alpha and beta
	return gtf.translation(image, alpha, beta) 



#histogram.py

#import dependencies
import numpy as np
import math
__all__ = ['eq_hist', 'histogram']

"""
eq_hist - function to equalize histogram
Parameters:
	hist		I/P	histogram of image
	colors		I/P	possible colors of image
	img_size	I/P	size of image
	equalized_hist	O/P	equalized histogram
"""
def eq_hist(hist, colors, img_size):
	hist = hist * colors / (2 * img_size)
	cum_hist = hist.cumsum() #Cummulative histogram
	equalized_hist = np.floor(cum_hist)

	return equalized_hist

"""	
histogram - function to comptue histogram of an image
Parameters:
	img_channel	I/P	image channel input in default/numpy array
	hist		O/P	computed histogram by numpy
"""
def histogram(img_channel):
	#hist, _ = np.histogram(im_channele, bins = 256, range = (0, 256))	
	hist = np.array([0] * 256)

	height = len(img_channel)
	width = len(img_channel[0])
	for row in range(height):
		for col in range(width):
			hist[img_channel[row, col]] += 1
			 
	return hist



#geom_transform.py
"""
Module: geom_transform - implement geometric transformation
Last updated: 02/02/2020
"""

#import dependencies
import numpy as np
__all__ = ['translation']

"""
translation - function to move the image with alpha and beta spaces
Parameters:
	image	I/P	the image input
	alpha	I/P	the num of spaces to move images horizontally
	beta	I/P	the num of spaces to move images vertically
	new_img	O/P	the moved image
"""
def translation(image, alpha, beta):
	#check alpha and beta in image range
	try:
		if alpha > image.shape[1] or beta > image.shape[0]:
			 raise NameError("Translation Error")
	except NameError:
		print('Alpha of ' + str(alpha) or ' Beta of ' + str(beta) + ' is out of image shape.')

	new_img = np.array([[255] * image.shape[1]] * image.shape[0])

	height = new_img.shape[0]
	width = new_img.shape[1]
	for row in range(height):
		if row + alpha in range(height):
			for col in range(width):
				if col + beta in range(width):
					new_img[row, col] = image[row + alpha, col + beta] 

	return new_img



#__init__.py of MoCV

print("Importing MoCV")

#import all modules of MoCV
__all__ = ['contrastImage',
	'histogram',
	'linear_scale_image',
	'segmentation',
	'geom_transform',
	'image_function']
from . import *



import setuptools

with open("README.md") as fh:
	long_description = fh.read()
	
setuptools.setup(
	name = "MoCV",
	version = "0.1.0",
	author = "datqngo",
	author_email = "quocdat32461997@yahoo.com",
	description = "List and details of Computer Vision algorithms",
	long_description = long_description,
	long_description_content_type = "text/markdown",
	url = "https://github.com/quocdat32461997/MoCV",
	packages = setuptools.find_packages(),
	classifiers = [
		"Programming Language :: Python :: 3",
        	"License :: OSI Approved :: MIT License",
        	"Operating System :: OS Independent",],
	install_requires = [
		"numpy",
		"matplotlib",
		"opencv-python"],
)



#segmentation.py
"""
	In Computer Vision, segmentation objects and background could be done by different methods:
		Intensity-based Segmentation: Thresholding - Based on the intensity of colors in histogram
		Edge-based Segmentation - Based on edges
		Region-based Segmentation
"""

#import dependencies
from . import histogram
import numpy as np
__all__ = ['optimal_thresholding']


"""
optimal_thresholding - function to segment objects and background using Quantization 
Parameters:
	image	I/P	image
	
"""
def optimal_thresholding(image):
	#calculate histogram
	hist = histogram.histogram(image)

	#thresholding by quantization
	optimal_thd = _quantization(hist)

	#segment image based on optimal_threshold
	front = np.array([[0] * image.shape[1]] * image.shape[0])
	back = np.array([[255] * image.shape[1]] * image.shape[0])

	for row in range(image.shape[0]):
		for col in range(image.shape[1]):
			if image[row, col] > optimal_thd:
				front[row, col] = image[row, col]
			else:
				back[row, col] = image[row, col]
	#reconstruct
	return front, back

#################### helper functions ###########################

"""
_quantization - private function to perform quanization (color compression) to find the optimal threshold within the image histogram
Parameters:
	hist		I/P	image histogram
	optimal_thd	O/P	optimal_threshold
"""
def _quantization(hist):
	#by default, histogram has 0-255 brightness levels
	min_error = -1
	optimal_threshold = -1	

	#Calculate q1 and q2 by calculating means of two classes of threshold
	q2_num = sum([k * hist[k] for k in range(len(hist))]) #total of #_pixels * color 
	q2_denom = sum(hist) #total of #_pixels
	q1_num = 0
	q1_denom = 0

	#Quantization, assume all brightness levels as possible thresholds
	for t in range(len(hist)) :
		q1 = 0
		q2 = 0

		#calculate q1
		if q1_denom == 0:
			q1 = 0
		else:
			q1 = np.floor(q1_num / q1_denom)

		#calculate q2
		if q2_denom == 0:
			q2 = 0
		else:
			q2 = np.floor(q2_num / q2_denom)

		#update numerator and denominator 
		q1_num += t * hist[t]
		q2_num -= t * hist[t]
		q1_denom += hist[t]
		q2_denom -= hist[t]

		#calculate threshold by calculating total varaince two classes
		threshold = int(np.mean([q1, q2]))
		temp_error = _calculate_error(q1, q2, threshold, hist)
		if optimal_threshold < 0: #if optimal_threshold not initialized
			min_error = temp_error
			optimal_threshold = threshold
		elif temp_error < min_error: #check the next min error to find the optimal threshold
			min_error = temp_error
			optimal_threshold = threshold

	return optimal_threshold

"""
_calculate_error - function to calculate sum of variance of two classes for error at a threshold
Parameters:
	q1		I/P	class 1
	q2		I/P	class 2
	threshold	I/P	threshold
	hist		I/P	histogram
	error		O/P	total error: q1_variance + q2_variance
"""
def _calculate_error(q1, q2, threshold, hist):
	#calclulate sum of variance of class 1
	q1_variance = 0
	for color in range(threshold):
		q1_variance += hist[color] * ((color - q1) ** 2)

	#calculate sum of variance of class 2
	q2_variance = 0
	for color in range(threshold, 256):
		q2_variance += hist[color] * ((color - q2) ** 2)

	return q1_variance + q2_variance
